import java.util.*;
import java.io.*;
import java.util.StringTokenizer;
import java.lang.*;

class Edge  {
	
	public Vertex toVertex;
	public  double weight;
	
	public Edge ( Vertex toVertex, double weight ) {
		this.toVertex = toVertex;
		this.weight = weight;
	}
	
}


class Vertex  {

	public String name;
	public double x, y;
	public LinkedList edgeList;
	//private Iterator it;
	
	public Vertex ( String name, double x, double y ) {
		this.name = name;
		this.x = x;
		this.y = y;
		edgeList = new LinkedList();
		//it = edgeList.iterator();
	}
	
	public Vertex ( String name ){
		this.name = name;
		this.x = 0;
		this.y = 0;
	}
	
	public void addEdge ( Vertex toVertex, double weight ) {
		edgeList.addLast ( new Edge ( toVertex, weight ) );
	}
	
	public void display() {
		if ( edgeList != null ) {
			Iterator it = edgeList.iterator(); 
			while ( it.hasNext() ) {
				Edge tempEdge = (Edge) (it.next());
				System.out.println( "     edge to: " + tempEdge.toVertex.name + ", wt = " + tempEdge.weight );
			}
		}
	}
}




class Digraph {

	public LinkedList vertexList;
	private Iterator it;
	final double Infinity = 1000000;
	public int size = 999;
	public double[] x = new double[size];
	public double[] y = new double[size];
	public int link[][] = new int[size][2];
	public int idxlink;
	public int ri=0;
	public String[] route = new String[size];
	public double min = 0;
	
	public Digraph() {
		vertexList = new LinkedList();
		it = vertexList.iterator();
	}
	
	public void readGraph( String fileName ) {
		//int size = 999;
		x = new double[size];
		y = new double[size];
		boolean[] linkto = new boolean[size];
		
		try {
			BufferedReader in = new BufferedReader( new FileReader( fileName ) );
			Object ref = null;
					
			ref = in.readLine();
			if (ref != null) {
					String line = new String ( (String) ref );
					StringTokenizer words = new StringTokenizer (line);
					size = Integer.parseInt(words.nextToken());
					
			}
			
			
			
			for ( int i = 1; i<=size; i++) {
			
				ref = in.readLine();
				if ( ref != null ) {
					String line = new String ( (String) ref );
					StringTokenizer words = new StringTokenizer (line);
										
						words = new StringTokenizer (line);
						double a = Double.parseDouble(words.nextToken());
						
					 	double b = Double.parseDouble(words.nextToken());
					 	
						x[i] = a;
						y[i] = b;
						linkto[i]=false;
						
					
				}
				
			} ;
			
			idxlink=0;		
			do {
				ref = in.readLine();
				if ( ref != null ) {
					String line = new String ( (String) ref );
					StringTokenizer words = new StringTokenizer (line);		
					//add links					
					String from = words.nextToken();
					int fromnode = Integer.parseInt(from);
					String to = words.nextToken();
					int tonode = Integer.parseInt(to);
					//int wt = Integer.parseInt( words.nextToken() );
					double wt = Math.sqrt( Math.pow((x[fromnode]-x[tonode]), 2.0) +
					 		   Math.pow((y[fromnode]-y[tonode]), 2.0) );
					
					//add edge					
					addEdge(from, x[fromnode], y[fromnode], to, x[tonode], 
							y[tonode], wt);
					linkto[fromnode]=true;
					linkto[tonode]=true;
					link[idxlink][0]=fromnode;
					link[idxlink][1]=tonode;
					idxlink=idxlink+1;
									
				}
			} while (ref != null);
		
			in.close();
			
			//check if all points link
			for ( int j=1; j<=size; j++) {
				if (! linkto[j]) {
					System.out.println("point " + j + " does not connect!");
					System.exit(1);
				}
			}
			
		} catch ( FileNotFoundException e ) {
			System.out.println("can't find file \"" + fileName+ "\"");
		} catch ( IOException e ) {
			System.out.println( "readLine() error" );
		}
	}
	
	public void processQuery( String fileName ) {
		try {
			BufferedReader in = new BufferedReader( new FileReader( fileName ) );
			Object ref = null;
			do {
				ref = in.readLine();
				if ( ref != null ) {
					String line = new String ( (String) ref );
					StringTokenizer words = new StringTokenizer (line);
					String from = words.nextToken();
					String to = words.nextToken();
					
					//add edge					
					Dijkstra(from, to);
				}
			} while (ref != null);
		
			in.close();
			
		} catch ( FileNotFoundException e ) {
			System.out.println("can't find file \"" + fileName+ "\"");
		} catch ( IOException e ) {
			System.out.println( "readLine() error" );
		}
	
	}
	
	public void processQuery ( String from, String to) {
		Dijkstra(from, to);
	}
	
		
	public void addEdge ( String from, double fromx, double fromy, String to, double tox, 
	                                  double toy, double weight ) {
		
		Vertex fromVertex = new Vertex ( from, fromx, fromy );
		Vertex toVertex = new Vertex ( to, tox, toy );
				
		if (vertexList == null ) { // add vertex at root
			vertexList.addLast ( fromVertex );
			vertexList.addLast ( toVertex );
			fromVertex.addEdge ( toVertex, weight );
		}
		else {
			Iterator it = vertexList.iterator();
			boolean addFrom = false;
			boolean addTo = false;
			
			while ( it.hasNext() && ( !addFrom || !addTo ) ) {
				Vertex tempVertex = (Vertex) (it.next());
				if (tempVertex.name.equals ( from ) )  { // fromVertex already exists, add edge
					tempVertex.addEdge( toVertex, weight );
					addFrom = true;
				} 
				else if ( tempVertex.name.equals ( to ) )  // toVertex already exists
					addTo = true;
			}
		 
			if ( addFrom == false ) {  // add fromVertex, add edge 
				vertexList.addLast ( fromVertex );	
				fromVertex.addEdge ( toVertex, weight );
			}
			
			if ( addTo == false )  // add toVertex
				vertexList.addLast ( toVertex );
		}
	}
	
	
	public void display() {
		if ( vertexList == null ) //graph is empty
			System.out.println( "The graph is empty" );
		
		else { //print the graph
			Iterator it = vertexList.iterator();
			
			while ( it.hasNext() ) {
				Vertex tempVertex = (Vertex) (it.next());
				System.out.println ( "Vertex: " +  tempVertex.name );
				tempVertex.display();
			}
		}
	}
	
	
	public void Dijkstra ( String source, String to ) {
		LinkedList s = new LinkedList();
		
		
		// step1: put the source vertex in S
		boolean addSource = false;
		
		if ( vertexList == null ) // graph is empty
			System.out.println( "The graph is empty");
		
		else {
			it = vertexList.iterator();
			Iterator edgeIt;
			Vertex sourceVertex = new Vertex( source );
			
			while ( it.hasNext() && ! addSource ) {
				Vertex tempVertex = (Vertex)(it.next());
				if ( source.equals( tempVertex.name ) ) { //put source in S
					s.addLast( sourceVertex );
					addSource = true;
				}
			}
			
			if ( !addSource ) // source not in graph
				System.out.println( source + "is not in the graph" ); 
			
			else { 
				
				// step2: for each vertex in vertexList, put in weights and predecessor
				HashMap weights = new HashMap( vertexList.size() );
				HashMap pre = new HashMap( vertexList.size() );
				it = vertexList.iterator();
				
				while ( it.hasNext() ) {
					Vertex tempVertex = (Vertex) ( it.next() );
					double weight = getWeight( sourceVertex, tempVertex );
					
					if ( weight == Infinity ) { // not adjancent to source
						weights.put( tempVertex, new Double (weight) );
						pre.put ( tempVertex, null );
					}
					else { //adjancent to source
						weights.put( tempVertex, new Double (weight) );
						pre.put( tempVertex, sourceVertex );
					}
					
				}
				
				//step3: 
				for ( int pass = 1; pass < vertexList.size(); pass++ ) {
					//add vertex that has min cost to S
					Vertex minVertex = minimun( s, weights );
					s.addLast( minVertex );
					it = vertexList.iterator();
					
					//exam the cost
					while ( it.hasNext() ) {
						Vertex tempVertex = (Vertex) ( it.next() );
						
						if ( !inS( tempVertex, s ) ) {
							double d1 = ( (Double) weights.get( tempVertex ) ).doubleValue();
							double d2 = ( (Double) weights.get ( minVertex ) ).doubleValue()  + getWeight( minVertex, tempVertex );
							if ( d2 < d1 ) { //change cost and predeceser
								weights.put ( tempVertex, new Double(d2) );
								pre.put( tempVertex, minVertex ); 
							}
						}
					}
				}
				
				//display query result
				display( source, to, weights, pre );
			}
		}
	}
	
	
	private void display ( String source, String to, HashMap weights, HashMap pre ) {
		Iterator it = vertexList.iterator();
		
		
		//find the min cost
		while ( it.hasNext() ) {
			Vertex tempVertex = (Vertex) it.next();
			if ( tempVertex.name.equals(to) )
				min =  ( (Double) weights.get(tempVertex) ).doubleValue();
		}
		
		//print the optiaml path		
		System.out.println();
		if ( min == Infinity ) 
			System.out.println("there is no path from " + source + " to " + to );
		else {
			Stack stack = new Stack();
			String temp = to;
			while ( !temp.equals( source ) ) {
				stack.push(temp);
				
				it = vertexList.iterator();
				boolean found = false;
				while ( it.hasNext() && !found ) {
					Vertex tempVertex = (Vertex) ( it.next() );
					if ( tempVertex.name.equals(temp) ) {
						temp = ( (Vertex) pre.get(tempVertex) ).name;
						found = true;
					}
				}
			}
			
			stack.push(temp);
			//System.out.println("an optimal path from " + source + " to " + to + " is:" );
			
			////
			///route = new String[999];
			
			
			temp = (String) stack.pop();
			while ( !temp.equals(to) ) {
				//System.out.print(temp);
				///
				route[ri]=temp;
				ri=ri+1;
				///
				//System.out.print("->");
				temp = (String) stack.pop();
			}
			
			System.out.println(temp);
			
			///
			route[ri]=temp;
			//System.out.println("cost is " + min );
			
			///
			for ( int tempi=0; tempi<=ri; tempi++) {
				System.out.println(route[tempi]);
			}
		}	 
	}
	
	
	private double getWeight( Vertex from, Vertex to ) {
		Iterator it = vertexList.iterator();
		
		while ( it.hasNext() ) {
			Vertex tempVertex = (Vertex) ( it.next() );
			
			if ( tempVertex.name.equals(from.name) ) {
				Iterator edgeIt = tempVertex.edgeList.iterator();
				
				while ( edgeIt.hasNext() ) {
					Edge tempToEdge = (Edge) ( edgeIt.next() );
					if ( tempToEdge.toVertex.name.equals( to.name ) )
						return tempToEdge.weight;
				}
				
				return Infinity; // not adjacent
			}
		}
		return Infinity;
	}
	
	
	private Vertex minimun( LinkedList s, HashMap weights ) {
		Iterator it = vertexList.iterator();
		double min = Infinity;
		Vertex minVertex = null;
					
		while ( it.hasNext() ) {
			Vertex tempVertex = (Vertex) ( it.next() );
								
			if ( !inS( tempVertex, s ) ) {
				if ( min >= ( (Double) weights.get( tempVertex ) ).doubleValue() ) {
					min = ( (Double) weights.get(tempVertex) ).doubleValue();
					minVertex = tempVertex;
				}
			}		
		}
		
		return minVertex;	
	}
	
	
	private boolean inS ( Vertex tempVertex, LinkedList s ) {
		Iterator sIt = s.iterator();
		boolean inS = false;
			
		while ( sIt.hasNext() && !inS ) { // check in S or not
			Vertex tempSVertex = (Vertex) ( sIt.next() );
			if ( tempVertex.name.equals(tempSVertex.name) )
				inS = true;
		}
		return inS;
	}
	

}